@@ -18,32 +18,39 @@ SEXP duplist(SEXP l, SEXP ans, SEXP anslen, SEXP order, SEXP tol)  //change name
     // UNIX uniq as referred to by ?base::unique; i.e., it drops immediately
     // repeated rows but doesn't drop duplicates of any previous row.
     // Unless, order is provided, then it also drops any previous row.
-    // l must be a list of same length integer vectors
+    // l must be a list of same length vectors
     // ans is allocated first (maximum length the number of rows) and the length returned in anslen.
+    // TO DO: grow ans instead.
     R_len_t i,j,nrow,ncol,len,thisi,previ;
+    Rboolean b,byorder;
+    SEXP v;
+    if (NA_INTEGER != NA_LOGICAL || sizeof(NA_INTEGER)!=sizeof(NA_LOGICAL)) error("Have assumed NA_INTEGER == NA_LOGICAL (currently R_NaInt). If R changes this in future (seems unlikely), an extra case is required; a simple change.");
     ncol = length(l);
     nrow = length(VECTOR_ELT(l,0));
     len=1;
     INTEGER(ans)[0] = 1;    // first row is always the first of the first group
-    if (INTEGER(order)[0] == -1) {      // i.e. order is missing, same order as row order. Using MISSING() does not seem stable under windows. Always having arguments passed in seems a good idea anyway.
-        for (i=1; i<nrow; i++) {
-            j = ncol;  // the last columns varies the most frequently so check that first and work backwards
-            while (--j>=0 && ( isReal(VECTOR_ELT(l,j)) ? 
-                               cmpnum(REAL(VECTOR_ELT(l,j))[i],REAL(VECTOR_ELT(l,j))[i-1]) :
-                               INTEGER(VECTOR_ELT(l,j))[i]==INTEGER(VECTOR_ELT(l,j))[i-1] ));
-     // INTEGER() will be used here for LOGICAL too (logical now allowed in keys and 'by' from Aug 2010) [ok now, but not future proof] TO DO: revisit
-            if (j>=0) INTEGER(ans)[len++] = i+1;
-        }
-    } else {
-        for (i=1; i<nrow; i++) {
-            j = ncol;
-            thisi = INTEGER(order)[i]-1;
-            previ = INTEGER(order)[i-1]-1;
-            while (--j>=0 && ( isReal(VECTOR_ELT(l,j)) ? 
-                               cmpnum(REAL(VECTOR_ELT(l,j))[thisi],REAL(VECTOR_ELT(l,j))[previ]) :
-                               INTEGER(VECTOR_ELT(l,j))[thisi]==INTEGER(VECTOR_ELT(l,j))[previ] ));
-            if (j>=0) INTEGER(ans)[len++] = i+1;
+    byorder = INTEGER(order)[0] != -1;
+    // Using MISSING() does not seem stable under windows. Always having arguments passed in seems a good idea anyway.
+    thisi = byorder ? INTEGER(order)[0]-1 : 0;
+    for (i=1; i<nrow; i++) {
+        previ = thisi;
+        thisi = byorder ? INTEGER(order)[i]-1 : i;
+        j = ncol;  // the last column varies the most frequently so check that first and work backwards
+        b = TRUE;
+        while (--j>=0 && b) {
+            v=VECTOR_ELT(l,j);
+            switch (TYPEOF(v)) {
+            case INTSXP : case LGLSXP :
+                b=INTEGER(v)[thisi]==INTEGER(v)[previ]; break;
+            case STRSXP :
+                b=STRING_ELT(v,thisi)==STRING_ELT(v,previ); break;
+            case REALSXP :
+                b=cmpnum(REAL(v)[thisi],REAL(v)[previ]); break;
+            default :
+                error("Type '%s' not supported", type2char(TYPEOF(v))); 
+            }
         }
+        if (!b) INTEGER(ans)[len++] = i+1;
     }
     INTEGER(anslen)[0] = len;
     return(R_NilValue);
